﻿//-----------------------------------------------------------------------
// <copyright file="GuessGameProvider.cs" company="BME MIT" />
//-----------------------------------------------------------------------
/*
 * Name:    IRF.ConsoleGuessGame.GuessGameProvider
 * Author:  Zoltan Micskei, BME MIT
 * Date:    2011.04.17.
 * Desc:    ConsoleGuessGame sample application for demonstrating manageable applications 
 *          see: http://www.inf.mit.bme.hu/edu/bsc/irf
 *          
 *          Decoupled, multi-instance WMI provider for the application named IRF_GuessGame. 
 *          Implements the properties and methods accessible from WMI.
 *          
 * Param:   name of the game session, which will be the key for the WMI instance
 */

// specify the WMI namespace and hosting model for the WMI provider
[assembly: System.Management.Instrumentation.WmiConfiguration(
    "root/irf", 
    HostingModel = System.Management.Instrumentation.ManagementHostingModel.Decoupled, 
    IdentifyLevel = false)]

namespace IRF.ConsoleGuessGame
{
    using System.Diagnostics;
    using System.Management.Instrumentation;

    using IRF.ConsoleGuessGame.Properties;

    /// <summary>
    /// Decoupled, multi-instance WMI provider for the application.
    /// </summary>
    [ManagementEntity(Name = "IRF_GuessGame")]
    [ManagementQualifier("Description", Value = "Provider for the GuessGame application.")]
    public class GuessGameProvider
    {        
        /// <summary>
        /// Name of the game session
        /// </summary>
        private string sessionName;

        /// <summary>
        /// Initializes a new instance of the GuessGameProvider class.
        /// </summary>
        /// <param name="sessionName">Name of the game session that will be used as the key of the WMI instance.</param>
        public GuessGameProvider([ManagementName("SessionName")] string sessionName)
        {
            Program.Log(Resources.ProviderConstructorLogMessage, LogCategories.Provider, LogEventIds.ProviderConstructorCalled, TraceEventType.Verbose);

            this.SessionName = sessionName;
        }

        /// <summary>
        /// Gets the name of the game session which is the key of this WMI provider.
        /// </summary>
        [ManagementKey]
        [ManagementQualifier("Description", Value = "The key of the instance, the name of the current guess game session.")]
        public string SessionName
        {
            get
            {
                using (Program.Tracer.StartTrace(LogCategories.Tracing))
                {
                    return this.sessionName;
                }
            }

            private set
            {
                this.sessionName = value;
            }
        }

        /// <summary>
        /// Gets the TotalGuesses WMI metric 
        /// </summary>
        [ManagementQualifier("Description", Value = "Returns the total number of guesses in this round (which were integer numbers).")]
        [ManagementProbe]
        public int TotalGuesses 
        {
            get
            {
                using (Program.Tracer.StartTrace(LogCategories.Tracing))
                {
                    if (Program.Game != null)
                    {
                        return Program.Game.TotalGuesses;
                    }
                    else
                    {
                        Program.Log(Resources.ProviderCouldNotAccessGameLogMessage, LogCategories.Provider, LogEventIds.ProviderCouldNotAccessGame, TraceEventType.Error);
                        return 0;
                    }    
                }                                
            }
        }

        /// <summary>
        /// Gets the TotalFaultyGuesses WMI metric 
        /// </summary>
        [ManagementProbe]
        [ManagementQualifier("Description", Value = "Returns the total number of guesses in this round, which were integer numbers, but were not in the accepted range.")]
        public int TotalFaultyGuesses
        {
            get
            {
                using (Program.Tracer.StartTrace(LogCategories.Tracing))
                {
                    if (Program.Game != null)
                    {
                        return Program.Game.TotalFaultyGuesses;
                    }
                    else
                    {
                        Program.Log(Resources.ProviderCouldNotAccessGameLogMessage, LogCategories.Provider, LogEventIds.ProviderCouldNotAccessGame, TraceEventType.Error);
                        return 0;
                    }    
                }                
            }
        }
        
        /// <summary>
        /// Starts a new round with calling the NewRound() method of GuessGame
        /// </summary>
        [ManagementTask]
        [ManagementQualifier("Description", Value = "Starts a new round of the game by chosing a new number and resetting the counters.")]
        public void NewRound()
        {
            using (Program.Tracer.StartTrace(LogCategories.Tracing))
            {
                if (Program.Game == null)
                {
                    Program.Log(Resources.ProviderCouldNotAccessGameLogMessage, LogCategories.Provider, LogEventIds.ProviderCouldNotAccessGame, TraceEventType.Error);
                }
                else
                {
                    Program.Game.NewRound();
                }
            }          
        }
    }
}
